@@ -35,7 +35,8 @@ impl<T: Send> OneSender<T> {
 impl<T: Send> Drop for OneSender<T> {
     fn drop(&mut self) {
         self.std_sender.take();
-        if let Some(waker) = self.inner.lock().unwrap().receiver_waker.take() {
+        let opt_waker = self.inner.lock().unwrap().receiver_waker.take();
+        if let Some(waker) = opt_waker {
             waker.wake();
         }
     }
@@ -63,11 +64,8 @@ impl<T: Send> Future for SendFut<T> {
             Err(TrySendError::Disconnected(value)) => Poll::Ready(Err(SendError(value))),
             Err(TrySendError::Full(value)) => {
                 self.value.set(Some(value));
-                self.inner
-                    .lock()
-                    .unwrap()
-                    .sender_wakers
-                    .push(cx.waker().clone());
+                let waker = cx.waker().clone();
+                self.inner.lock().unwrap().sender_wakers.push(waker);
                 Poll::Pending
             }
         }
@@ -105,7 +103,8 @@ impl<T: Send + Clone> SyncSender<T> {
 }
 impl<T: Send> SyncSender<T> {
     fn wake_receiver(&self) {
-        if let Some(waker) = self.inner.lock().unwrap().receiver_waker.take() {
+        let opt_waker = self.inner.lock().unwrap().receiver_waker.take();
+        if let Some(waker) = opt_waker {
             waker.wake();
         }
     }
@@ -171,12 +170,12 @@ pub struct Receiver<T>
 where
     T: Send,
 {
-    std_receiver: std::sync::mpsc::Receiver<T>,
+    std_receiver: Option<std::sync::mpsc::Receiver<T>>,
     inner: Arc<Mutex<Inner>>,
 }
 impl<T: Send> Receiver<T> {
     fn wake_senders(&self) {
-        let wakers: Vec<Waker> = self.inner.lock().unwrap().sender_wakers.drain(..).collect();
+        let wakers: Vec<Waker> = std::mem::take(&mut self.inner.lock().unwrap().sender_wakers);
         for waker in wakers {
             waker.wake();
         }
@@ -218,8 +217,9 @@ impl<T: Send> Receiver<T> {
     /// Compared with `recv`, this function has two failure cases
     /// instead of one (one for disconnection, one for an empty buffer).
     #[allow(clippy::missing_errors_doc)]
+    #[allow(clippy::missing_panics_doc)]
     pub fn try_recv(&self) -> Result<T, std::sync::mpsc::TryRecvError> {
-        self.wake_senders_if_ok(self.std_receiver.try_recv())
+        self.wake_senders_if_ok(self.std_receiver.as_ref().unwrap().try_recv())
     }
 
     /// Attempts to wait for a value on this receiver, returning an error if the corresponding
@@ -235,8 +235,9 @@ impl<T: Send> Receiver<T> {
     /// received on this channel.  However, since channels are buffered, messages sent before the
     /// disconnect will still be properly received.
     #[allow(clippy::missing_errors_doc)]
+    #[allow(clippy::missing_panics_doc)]
     pub fn recv(&self) -> Result<T, std::sync::mpsc::RecvError> {
-        self.wake_senders_if_ok(self.std_receiver.recv())
+        self.wake_senders_if_ok(self.std_receiver.as_ref().unwrap().recv())
     }
 
     /// Attempts to wait for a value on this receiver, returning an error if the corresponding
@@ -256,11 +257,12 @@ impl<T: Send> Receiver<T> {
     /// that can cause `recv_timeout` to panic unexpectedly.  See the explanation at
     /// [`std::sync::mpsc::Receiver::recv_timeout`](https://doc.rust-lang.org/std/sync/mpsc/struct.Receiver.html#method.recv_timeout).
     #[allow(clippy::missing_errors_doc)]
+    #[allow(clippy::missing_panics_doc)]
     pub fn recv_timeout(
         &self,
         timeout: core::time::Duration,
     ) -> Result<T, std::sync::mpsc::RecvTimeoutError> {
-        self.wake_senders_if_ok(self.std_receiver.recv_timeout(timeout))
+        self.wake_senders_if_ok(self.std_receiver.as_ref().unwrap().recv_timeout(timeout))
     }
 
     /// Attempts to wait for a value on this receiver, returning an error if the corresponding
@@ -280,7 +282,7 @@ impl<T: Send> Receiver<T> {
         &self,
         deadline: std::time::Instant,
     ) -> Result<T, std::sync::mpsc::RecvTimeoutError> {
-        self.wake_senders_if_ok(self.std_receiver.recv_deadline(deadline))
+        self.wake_senders_if_ok(self.std_receiver.as_ref().unwrap().recv_deadline(deadline))
     }
 
     /// Returns an iterator that will block waiting for messages, but never panic.
@@ -298,7 +300,9 @@ impl<T: Send> Receiver<T> {
 }
 impl<T: Send> Drop for Receiver<T> {
     fn drop(&mut self) {
-        self.inner.lock().unwrap().receiver_waker.take();
+        let waker = self.inner.lock().unwrap().receiver_waker.take();
+        drop(waker);
+        self.std_receiver.take();
         self.wake_senders();
     }
 }
@@ -307,11 +311,12 @@ impl<T: Send> Future for Receiver<T> {
     type Output = Result<T, std::sync::mpsc::RecvError>;
 
     fn poll(self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<Self::Output> {
-        match self.std_receiver.try_recv() {
+        match self.std_receiver.as_ref().unwrap().try_recv() {
             Ok(value) => Poll::Ready(Ok(value)),
             Err(TryRecvError::Disconnected) => Poll::Ready(Err(RecvError)),
             Err(TryRecvError::Empty) => {
-                self.inner.lock().unwrap().receiver_waker = Some(cx.waker().clone());
+                let waker = cx.waker().clone();
+                let _opt_waker = self.inner.lock().unwrap().receiver_waker.replace(waker);
                 Poll::Pending
             }
         }
@@ -420,7 +425,7 @@ where
             inner: inner.clone(),
         },
         Receiver {
-            std_receiver,
+            std_receiver: Some(std_receiver),
             inner,
         },
     )
@@ -466,7 +471,7 @@ where
             inner: inner.clone(),
         },
         Receiver {
-            std_receiver,
+            std_receiver: Some(std_receiver),
             inner,
         },
     )
